using System.Collections.Generic;
using System.Runtime.Serialization;

namespace JamesKovacs.Examples.WcfCircularRefs.DTO {
    [DataContract(Namespace="http://www.jameskovacs.com/Examples/WcfCircularRefs")]
    public class PersonDTO {
        private static readonly PersonDTO s_NullPersonDTO = new PersonDTO("Unknown");

        public PersonDTO(string name) : this(name, null) {
        }

        private PersonDTO(string name, PersonDTO parent) {
            m_name = name;
            m_parent = parent ?? s_NullPersonDTO;
        }

        [DataMember(Order = 1, IsRequired = true)]
        public string Name {
            get { return m_name; }
            set { m_name = value; }
        }

        public IList<PersonDTO> Children {
            get { return m_children.AsReadOnly(); }
        }

        public PersonDTO Parent {
            get { return m_parent; }
        }

        public void AddChild(string name) {
            PersonDTO child = new PersonDTO(name, this);
            m_children.Add(child);
        }

        public override string ToString() {
            return formatPerson(this, 0);
        }

        private string formatPerson(PersonDTO p, int indentLevel) {
            string indent = new string('\t', indentLevel);
            string retval = string.Format("{0}{1} - My parent is {2}.", indent, p.Name, p.Parent.Name);
            foreach(PersonDTO child in p.Children) {
                retval += formatPerson(child, indentLevel + 1);
            }
            return retval;
        }

        private string m_name;
        [DataMember(Name = "Parent", Order = 2, IsRequired = true)]
        private PersonDTO m_parent;
        [DataMember(Name = "Children", Order = 3, IsRequired = true)]
        private List<PersonDTO> m_children = new List<PersonDTO>();
    }
}
